from pathlib import Path

from glider_ingest import MissionData, MissionProcessor

memory_card_copy_loc = Path('path/to/memory/card/copy')
# Where you want the netcdf to be saved to
working_dir = Path('path/to/working/dir').resolve()
mission_num = '46'

# Initalize the mission_data container
mission_data = MissionData(memory_card_copy_loc=memory_card_copy_loc,
                         working_dir=working_dir,
                         mission_num=mission_num)
# Pass the mission_data container to the MissionProcessor class
# call save_mission_dataset to generate and save the mission dataset
MissionProcessor(mission_data=mission_data).save_mission_dataset()
